/***
 * Think ADL Parser
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: think@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.composite.c.controllers;

import static org.objectweb.fractal.adl.NodeUtil.castNodeError;

import java.util.List;
import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.ComponentVisitor;
import org.objectweb.fractal.adl.Node;
import org.objectweb.fractal.adl.bindings.Binding;
import org.objectweb.fractal.adl.bindings.BindingContainer;
import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.cecilia.adl.AbstractInstantiationTask;
import org.objectweb.fractal.cecilia.adl.SourceCodeProvider;
import org.objectweb.fractal.cecilia.adl.file.CodeWriter;
import org.objectweb.fractal.task.core.AbstractTaskFactoryUser;
import org.objectweb.fractal.task.core.TaskException;
import org.objectweb.fractal.task.core.primitive.annotations.ServerInterface;
import org.objectweb.fractal.task.core.primitive.annotations.ServerInterfaces;
import org.objectweb.fractal.task.core.primitive.annotations.TaskParameters;

/**
 * Visitor component that builds the instantiation of the binding controller
 * data structure of a default composite component.
 */
public class BCDynamicDescriptorVisitor extends AbstractTaskFactoryUser
    implements
      ComponentVisitor {

  // ---------------------------------------------------------------------------
  // Implementation of the ComponentVisitor interface
  // ---------------------------------------------------------------------------

  /**
   * Visits {@link ComponentContainer} nodes and creates a task that writes the
   * source code containing the initialization of the binding controller data
   * structure.
   */
  public Component visit(final List<Node> path,
      final ComponentContainer container, final Map<Object, Object> context)
      throws ADLException, TaskException {
    final BindingContainer bindingContainer = castNodeError(container,
        BindingContainer.class);
    return createTask(container, bindingContainer.getBindings());
  }

  // ---------------------------------------------------------------------------
  // Utility methods
  // ---------------------------------------------------------------------------

  protected Component createTask(final ComponentContainer container,
      final Binding[] bindings) throws TaskException {
    return taskFactoryItf.newPrimitiveTask(new BCInstantiationTask(bindings),
        container);
  }

  // ---------------------------------------------------------------------------
  // Task classes
  // ---------------------------------------------------------------------------

  /**
   * Builds source code for the instantiation of the binding controller data
   * structure for the given component node. This task provides the produced
   * source code.
   */
  @TaskParameters("componentNode")
  @ServerInterfaces(@ServerInterface(name = "bc-instantiation-provider", signature = SourceCodeProvider.class, record = "role:controllerInstantiation, id:%, controller:BC", parameters = "componentNode"))
  public static class BCInstantiationTask extends AbstractInstantiationTask {
    protected final Binding[] bindings;

    // -------------------------------------------------------------------------
    // Task constructor
    // -------------------------------------------------------------------------

    /**
     * @param bindings the binding nodes.
     */
    public BCInstantiationTask(final Binding[] bindings) {
      this.bindings = bindings;
    }

    // -------------------------------------------------------------------------
    // Implementation of abstract methods of AbstractInstantiationTask
    // -------------------------------------------------------------------------

    @Override
    protected String processSourceCode() throws Exception {
      final CodeWriter cw = new CodeWriter("Binding Controller Data Descriptor");

      final String instanceName = instanceNameProviderItf.getCInstanceName();

      cw.append("static binding_desc_t ").append(instanceName).append(
          "_bindings_desc[] = {").endl();

      for (final Binding binding : bindings) {
        final String from = binding.getFrom();
        int index = from.indexOf('.');
        String clientComp = from.substring(0, index);
        final String clientItf = from.substring(index + 1);

        final String to = binding.getTo();
        index = to.indexOf('.');
        String serverComp = to.substring(0, index);
        final String serverItf = to.substring(index + 1);

        if (serverComp.equals("this"))
          serverComp = "thisComposite";
        else if (clientComp.equals("this")) clientComp = "thisComposite";

        cw.append("{").append(clientComp.toString().replace('.', '_')).append(
            "_exp, \"").append(clientItf).append("\", ").append(
            serverComp.toString().replace('.', '_')).append("_exp, \"").append(
            serverItf).append("\"},").endl();

      }

      cw.append("};").endl();

      return cw.toString();
    }
  }
}
