/***
 * Think ADL Parser
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: think@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.primitive.thinkMC.attributes;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.fractal.adl.attributes.Attribute;
import org.objectweb.fractal.adl.attributes.Attributes;
import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.cecilia.adl.AbstractInstantiationTask;
import org.objectweb.fractal.cecilia.adl.SourceCodeProvider;
import org.objectweb.fractal.cecilia.adl.attributes.AbstractAttributeVisitor;
import org.objectweb.fractal.cecilia.adl.file.CodeWriter;
import org.objectweb.fractal.cecilia.adl.idl.ast.Field;
import org.objectweb.fractal.cecilia.adl.idl.ast.FieldContainer;
import org.objectweb.fractal.cecilia.adl.idl.ast.IDLDefinitionContainer;
import org.objectweb.fractal.cecilia.adl.idl.ast.Type;
import org.objectweb.fractal.cecilia.adl.idl.util.CUtil;
import org.objectweb.fractal.cecilia.adl.idl.util.Util;
import org.objectweb.fractal.task.core.TaskException;
import org.objectweb.fractal.task.core.primitive.annotations.ServerInterface;
import org.objectweb.fractal.task.core.primitive.annotations.ServerInterfaces;
import org.objectweb.fractal.task.core.primitive.annotations.TaskParameters;

/**
 * Visitor component that builds the instantiation of the attribute data
 * structures of the component for <code>ThinkMC</code> dialect.
 */
// TODO back port r3677:4017 from AttributeInstanceBuilder.java
public class AttributeDescriptorVisitor extends AbstractAttributeVisitor {

  // ---------------------------------------------------------------------------
  // Implementation of abstract methods of AbstractAttributeVisitor
  // ---------------------------------------------------------------------------

  @Override
  protected Component createTask(final ComponentContainer container,
      final Attributes attributes) throws TaskException {
    final Map<String, Type> fieldTypes = new HashMap<String, Type>();
    for (final Field field : ((FieldContainer) ((IDLDefinitionContainer) attributes)
        .getIDLDefinition()).getFields()) {
      fieldTypes.put(field.getName(), Util.getContainedType(field));
    }
    return taskFactoryItf.newPrimitiveTask(new AttributeDefinitionTask(
        attributes.getAttributes(), fieldTypes), container);
  }

  // ---------------------------------------------------------------------------
  // Task classes
  // ---------------------------------------------------------------------------

  /**
   * Builds definition source code for attribute data structure for the given
   * component node. This task provides the produced source code.
   */
  @TaskParameters("componentNode")
  @ServerInterfaces(@ServerInterface(name = "source-code-provider", signature = SourceCodeProvider.class, record = "role:attributeInstantiation, id:%", parameters = "componentNode"))
  public static class AttributeDefinitionTask extends AbstractInstantiationTask {

    protected final Map<String, Type> fieldTypes;
    // the attributes array
    protected final Attribute[]       fields;

    // -------------------------------------------------------------------------
    // Task constructor
    // -------------------------------------------------------------------------

    /**
     * @param fields the array of fields of the attribute data structure of the
     *            component.
     * @param fieldTypes the attribute types.
     */
    public AttributeDefinitionTask(final Attribute[] fields,
        final Map<String, Type> fieldTypes) {
      this.fields = fields;
      this.fieldTypes = fieldTypes;
    }

    // -------------------------------------------------------------------------
    // Implementation of abstract methods of AbstractInstantiationTask
    // -------------------------------------------------------------------------

    @Override
    protected String processSourceCode() throws Exception {
      final CodeWriter cw = new CodeWriter();
      final CodeWriter attr_cw = new CodeWriter();

      final String name = instanceNameProviderItf.getCInstanceName();

      // Build attribute_desc_t structures
      for (final Attribute attribute : fields) {
        final String attrName = attribute.getName().replace('.', '_');
        final String attrValue = attribute.getValue();
        final Type attType = fieldTypes.get(attrName);
        final String attTypeStr = CUtil.buildReturnType(attType);
        if (attrValue != null) {
          cw.append("static const ").append(
              attTypeStr + " " + name + "_" + attrName + "_attr_initvalue")
              .append(" = ");
          try {
            Integer.parseInt(attrValue);
            cw.append(attrValue);
          } catch (final NumberFormatException e) {
            if (attrValue.startsWith("0x") || attrValue.startsWith("0X"))
              cw.append(attrValue);
            else
              cw.append("\"").append(attrValue).append("\"");
          }
          cw.append(";").endl();
        }
        attr_cw.append("{").endl();
        attr_cw.append("\"").append(attrName).append("\", // name").endl();
        if (attrValue == null) {
          attr_cw.append("0");
        } else {
          attr_cw.append("(unsigned char *) &").append(name).append("_")
              .append(attrName).append("_attr_initvalue");
        }
        attr_cw.append(", // initialValues").endl();

        attr_cw.append("sizeof(").append(attTypeStr).append("), // size")
            .endl();

        attr_cw.append("(unsigned int)&(((struct ").append(
            typeNameProviderItf.getCTypeName()).append("_t *)0)->attributes.")
            .append(attrName).append(")").append("// offset").endl();
        attr_cw.append("},").endl();
      }

      // Build array of attribute_desc_t
      cw.append("static attribute_desc_t ").append(name).append(
          "_attributes_desc[]     ").append(" = {").endl();
      cw.append(attr_cw.toString());
      cw.append("};").endl();

      return cw.toString();

    }
  }
}
